package org.expeditee.io;

import java.awt.Desktop;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashMap;

import org.expeditee.agents.WriteTree;
import org.expeditee.gui.Browser;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameIO;
import org.expeditee.items.Item;
import org.expeditee.items.Line;
import org.expeditee.items.Text;

public abstract class DefaultFrameWriter extends ItemWriter implements
		FrameWriter {

	protected String _filename = null;

	protected String _output = null;

	protected ProxyWriter _writer = null;

	protected String _format = "";

	protected boolean _running = true;

	protected boolean _stop = false;

	// keep track of methods that are put on the same line
	protected static LinkedHashMap<String, Method> _ItemTags = null;

	protected static LinkedHashMap<String, Method> _FrameTags = null;

	public DefaultFrameWriter() {
		if (_ItemTags != null && _FrameTags != null)
			return;

		_ItemTags = new LinkedHashMap<String, Method>();
		_FrameTags = new LinkedHashMap<String, Method>();

		Class[] param = {};

		try {
			_FrameTags.put("V", Frame.class.getMethod("getVersion", param));
			_FrameTags.put("p", Frame.class.getMethod("getPermission", param));
			_FrameTags.put("U", Frame.class.getMethod("getOwner", param));
			_FrameTags.put("D", Frame.class.getMethod("getDateCreated", param));
			_FrameTags.put("M", Frame.class.getMethod("getLastModifyUser",
					param));
			_FrameTags.put("d", Frame.class.getMethod("getLastModifyDate",
					param));
			_FrameTags.put("F", Frame.class.getMethod("getFrozenDate", param));

			_FrameTags.put("O", Frame.class.getMethod("getForegroundColor",
					param));
			_FrameTags.put("B", Frame.class.getMethod("getBackgroundColor",
					param));
			_ItemTags.put("S", Item.class.getMethod("getTypeAndID", param));
			_ItemTags.put("s", Item.class.getMethod("getDateCreated", param));
			_ItemTags.put("d", Item.class.getMethod("getColor", param));
			_ItemTags.put("G", Item.class
					.getMethod("getBackgroundColor", param));
			_ItemTags.put("K", Item.class.getMethod("getBorderColor", param));
			_ItemTags.put("H", Item.class.getMethod("getAnchorRight", param));
			_ItemTags.put("I", Item.class.getMethod("getAnchorBottom", param));
			_ItemTags.put("P", Item.class.getMethod("getPosition", param));
			_ItemTags.put("F", Item.class.getMethod("getLink", param));
			_ItemTags.put("X", Item.class.getMethod("getAction", param));
			_ItemTags.put("x", Item.class.getMethod("getActionMark", param));
			_ItemTags.put("U", Item.class.getMethod("getActionCursorEnter",
					param));
			_ItemTags.put("V", Item.class.getMethod("getActionCursorLeave",
					param));
			_ItemTags.put("W", Item.class.getMethod("getActionEnterFrame",
					param));
			_ItemTags.put("Y", Item.class.getMethod("getActionLeaveFrame",
					param));
			_ItemTags.put("D", Item.class.getMethod("getData", param));
			_ItemTags.put("u", Item.class.getMethod("getHighlight", param));
			_ItemTags.put("e", Item.class.getMethod("getFillColor", param));
			_ItemTags.put("E", Item.class.getMethod("getGradientColor", param));

			_ItemTags.put("i", Item.class.getMethod("getFillPattern", param));
			_ItemTags.put("o", Item.class.getMethod("getOwner", param));
			_ItemTags.put("n", Item.class.getMethod("getLinkMark", param));
			_ItemTags.put("q", Item.class.getMethod("getLinkFrameset", param));
			_ItemTags.put("y", Item.class.getMethod("getLinkTemplate", param));
			_ItemTags.put("g", Item.class.getMethod("getLinePattern", param));

			_ItemTags.put("j", Item.class.getMethod("getArrow", param));

			_ItemTags.put("f", Text.class.getMethod("getFont", param));
			_ItemTags.put("t", Text.class.getMethod("getSpacing", param));

			// TODO set a boolean flag to indicate that the text is a formula
			// Store the formula in the text property NOT the answer
			_ItemTags.put("J", Item.class.getMethod("getFormula", param));
			_ItemTags.put("T", Text.class.getMethod("getText", param));

			_ItemTags.put("a", Text.class.getMethod("getWordSpacing", param));
			_ItemTags.put("b", Text.class.getMethod("getLetterSpacing", param));
			_ItemTags
					.put("m", Text.class.getMethod("getInitialSpacing", param));
			_ItemTags.put("w", Text.class.getMethod("getWidth", param));
			_ItemTags.put("k", Text.class.getMethod("getJustification", param));

			_ItemTags.put("h", Item.class.getMethod("getThickness", param));
			_ItemTags.put("l", Item.class.getMethod("getLineIDs", param));
			_ItemTags.put("c", Item.class.getMethod("getConstraintIDs", param));

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void setOutputLocation(String filename) {
		_filename = filename;
	}

	public String writeFrame(Frame toWrite) throws IOException {
		return writeFrame(toWrite, null);
	}

	public String writeFrame(Frame toWrite, Writer writer) throws IOException {
		try {
			initialise(toWrite, writer);

			outputFrame(toWrite);

			_running = false;
			return finaliseFrame();
		} catch (IOException ioe) {
			_running = false;
			throw ioe;
		}

	}

	/**
	 * Called before writing out the body items of each frame.
	 * 
	 * @param starting
	 *            the name of the frame currently being written out.
	 * @throws IOException
	 */
	protected void writeStartFrame(Frame starting) throws IOException {
		if (starting.getTitleItem() != null) {
			if (starting.getTitleItem().isAnnotation())
				this.writeAnnotationTitle(starting.getTitleItem());
			else
				this.writeTitle(starting.getTitleItem(), starting.getItems());
		}
	}

	/**
	 * Called after writing out the body items of each frame.
	 * 
	 * @param ending
	 *            the name of the frame currently being written out.
	 * @throws IOException
	 */
	protected void writeEndFrame(Frame ending) throws IOException {
	}

	protected final void initialise(Frame start) throws IOException {
		initialise(start, null);
	}

	protected void initialise(Frame start, Writer writer) throws IOException {
		if (_filename == null)
			_filename = FrameIO.EXPORTS_DIR + getFileName(start) + _format;

		if (writer != null) {
			_writer = new ProxyWriter(writer);
			_output = writer.toString();
		} else if (_filename.equalsIgnoreCase(WriteTree.CLIPBOARD)) {
			_writer = new ProxyWriter(true);
			_output = WriteTree.CLIPBOARD;
		} else {
			if (_filename.contains(File.separator)) {
				String extTest = _filename.substring(_filename
						.lastIndexOf(File.separator) + 1);
				if (!extTest.contains("."))
					_filename += _format;
			} else if (!_filename.contains("."))
				_filename += _format;

			if (!_filename.contains(File.separator))
				_filename = FrameIO.EXPORTS_DIR + _filename;

			File test = new File(_filename);

			test = test.getParentFile();
			if (test != null && !test.exists()) {
				test.mkdirs();
			}

			_writer = new ProxyWriter(new FileWriter(_filename));
			_output = _filename;
		}
	}

	protected String getFileName(Frame start) {
		return getValidFilename(start.getTitle());
	}

	public static String getValidFilename(String filename) {
		return filename.replaceAll("[ \\.]", "_");
	}

	protected String finalise() throws IOException {
		try {
			_writer.flush();
			_writer.close();
		} catch (IOException ioe) {
		} finally {
			_writer.close();
		}
		try {
			if (Browser._theBrowser.isVersion6()) {
				if (Desktop.isDesktopSupported()) {
					Desktop.getDesktop().open(new File(_output));
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return " exported to " + _output;
	}

	protected String finaliseFrame() throws IOException {
		return "Frame" + finalise();
	}

	protected void outputFrame(Frame toWrite) throws IOException {
		writeStartFrame(toWrite);

		Collection<Item> done = new HashSet<Item>();

		for (Item i : getItemsToWrite(toWrite)) {
			if (_stop) {
				return;
			}

			if (i instanceof Line) {
				if (done.contains(i)) {
					continue;
				}
				done.addAll(i.getAllConnected());
			}
			writeItem(i);
		}
		writeEndFrame(toWrite);
	}

	protected Collection<Item> getItemsToWrite(Frame toWrite) {
		return toWrite.getItemsToSave();
	}

	public boolean isRunning() {
		return _running;
	}

	public void stop() {
		_stop = true;
	}

	public String getFileContents() {
		return "Not Supported";
	}

}
